@@ -15,16 +15,25 @@
 #
 
 package MongoDB::Async::Code;
-{
-  $MongoDB::Async::Code::VERSION = '0.702.3';
-}
-
+our $VERSION = '0.45';
 
 # ABSTRACT: JavaScript Code
 
+=head1 NAME
 
-use Moose;
+MongoDB::Async::Code - JavaScript code
 
+=cut
+
+use Mouse;
+
+=head1 ATTRIBUTES
+
+=head2 code
+
+A string of JavaScript code.
+
+=cut
 
 has code => (
     is       => 'ro',
@@ -32,6 +41,11 @@ has code => (
     required => 1,
 );
 
+=head2 scope
+
+An optional hash of variables to pass as the scope.
+
+=cut
 
 has scope => (
     is       => 'ro',
@@ -40,57 +54,3 @@ has scope => (
 );
 
 1;
-
-__END__
-
-=pod
-
-=head1 NAME
-
-MongoDB::Async::Code - JavaScript Code
-
-=head1 VERSION
-
-version 0.702.3
-
-=head1 NAME
-
-MongoDB::Async::Code - JavaScript code
-
-=head1 ATTRIBUTES
-
-=head2 code
-
-A string of JavaScript code.
-
-=head2 scope
-
-An optional hash of variables to pass as the scope.
-
-=head1 AUTHORS
-
-=over 4
-
-=item *
-
-Florian Ragwitz <rafl@debian.org>
-
-=item *
-
-Kristina Chodorow <kristina@mongodb.org>
-
-=item *
-
-Mike Friedman <mike.friedman@10gen.com>
-
-=back
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is Copyright (c) 2012 by 10gen, Inc..
-
-This is free software, licensed under:
-
-  The Apache License, Version 2.0, January 2004
-
-=cut
