@@ -15,43 +15,13 @@
 #
 
 package MongoDB::Async::Timestamp;
-{
-  $MongoDB::Async::Timestamp::VERSION = '0.702.3';
-}
-
+our $VERSION = '0.45';
 
 # ABSTRACT: Replication timestamp
 
-
-use Moose;
-
-
-has sec => (
-    is       => 'ro',
-    isa      => 'Int',
-    required => 1,
-);
-
-
-has inc => (
-    is       => 'ro',
-    isa      => 'Int',
-    required => 1,
-);
-
-1;
-
-__END__
-
-=pod
-
 =head1 NAME
 
-MongoDB::Async::Timestamp - Replication timestamp
-
-=head1 VERSION
-
-version 0.702.3
+MongoDB::Async::Timestamp - Timestamp used for replication
 
 =head1 SYNOPSIS
 
@@ -60,9 +30,9 @@ times, or timestamps in the traditional sense.  Unless you are looking to mess
 with MongoDB's replication internals, the class you are probably looking for is
 L<DateTime>.  See <MongoDB::Async::DataTypes> for more information.
 
-=head1 NAME
+=cut
 
-MongoDB::Async::Timestamp - Timestamp used for replication
+use Mouse;
 
 =head1 ATTRIBUTES
 
@@ -70,34 +40,24 @@ MongoDB::Async::Timestamp - Timestamp used for replication
 
 Seconds since epoch.
 
+=cut
+
+has sec => (
+    is       => 'ro',
+    isa      => 'Int',
+    required => 1,
+);
+
 =head2 inc
 
 Incrementing field.
 
-=head1 AUTHORS
-
-=over 4
-
-=item *
-
-Florian Ragwitz <rafl@debian.org>
-
-=item *
-
-Kristina Chodorow <kristina@mongodb.org>
-
-=item *
-
-Mike Friedman <mike.friedman@10gen.com>
-
-=back
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is Copyright (c) 2012 by 10gen, Inc..
-
-This is free software, licensed under:
-
-  The Apache License, Version 2.0, January 2004
-
 =cut
+
+has inc => (
+    is       => 'ro',
+    isa      => 'Int',
+    required => 1,
+);
+
+1;
