@@ -16,7 +16,7 @@ use File::Slurp;
 use Tie::IxHash;
 use FileHandle;
 
-my $conn = MongoDB::Async::MongoClient->new("username" => "kristina", "password" => "foo", "db_name" => "bar", "ssl" => $ENV{MONGO_SSL});
+my $conn = MongoDB::Async::Connection->new("username" => "kristina", "password" => "foo", "db_name" => "bar");
 
 my $db = $conn->get_database("bar");
 my $c = $db->get_collection("x");
